/* 
   Hello World pour libNDS
   Auteur : Fares BELHADJ
   Email  : amsi@ai.univ-paris8.fr
   Date   : 02/10/2007
*/

#include <nds.h>
#include <stdio.h>
#include <stdlib.h>
#include "primitives.h"

static void init(void);
static void manageEvents(void);
static void messages(void);
static void follow(int x, int y);
static void cls(void);

/*
 * un tout petit main
 */
int main(void) {
  init();
  while(1) {
    messages();
    manageEvents();
  }
  return 0;
}

/*
 * Initialiser l'affichage de la console
 */
static void init(void) {
  consoleDemoInit();
  videoSetMode   (MODE_FB0);   /* direct frame buffer */
  vramSetBankA   (VRAM_A_LCD); /* quelle partie de la VRAM utiliser */
  lcdSwap(); /* echanger les deux ecrans d'affichage */
}

/*
 * Gestion des evenements
 */
static void manageEvents(void) {
  /* Position du stylet sur l'ecran tactile */
  touchPosition touchXY = touchReadXY();
  /* Recuperation des evenements */
  scanKeys();
  /* Si evenement lie a l'ecran tactile */
  /* Reinitialiser le chemin */
  if(keysUp() & KEY_TOUCH)
    follow(-1, -1);
  /* Dessiner */
  else if ((keysDown() | keysHeld()) & KEY_TOUCH)
    follow(touchXY.px, touchXY.py);
  /* Si la touche A a ete enfoncee */
  if(keysDown() & KEY_A) cls();
}

/*
 * Afficher les positions du stylet.
 */
static void messages(void) {
  touchPosition txy = touchReadXY();
  /*
   * Utilisation de la sequence d'echappement "\x1b[ligne;colonneH"
   * pour placer les caracteres 
   */
  iprintf("\x1b[0;0H");
  iprintf("{touche A = cls}");
  iprintf("\x1b[19;0H");
  iprintf("Hello World !\n");
  iprintf("Touch x = %06d, %06d\n", txy.x, txy.px);
  iprintf("Touch y = %06d, %06d\n", txy.y, txy.py);
}

/*
 * Dessiner un chemin continu entre les points successifs envoyes par
 * l'ecran tactile
 */
static void follow(int x, int y) {
  static int ox = -1, oy = -1;
  if(!x && !y) return; /* Bug ???? */
  setColor15(RGB15(31, 31, 31));
  if(!inScreen(ox, oy) || !inScreen(x, y)) { 
    ox = x; oy = y;
    vramaPutPixel(x, y);
    return; /* ca suffit */
  } 
  /* relier l'ancien et le nouveau point */
  drawLine(vramaPutPixel, x, y, ox, oy);
  /* preparer la prochaine etape */
  iprintf("(%4d %4d -> %4d %4d)\n", ox, oy, x, y);
  ox = x; oy = y; 
}

/*
 * effacer l'ecran (noir) 
 *
 * (a faire ? : integrer dans primitives.c avec la possibilite de
 * modifier la couleur de fond, ou transformer en fonction "rapide" de
 * remplissage de rectangle dans la VRAM)
 */
static void cls(void) {
  int i;
  for(i = 0; i < SCREEN_WIDTH * SCREEN_HEIGHT; i++)
    VRAM_A[i] = RGB15(0, 0, 0);
}
