/* 
 * Primitives de dessin pour libNDS
 * Auteur : Fares BELHADJ
 * Email  : amsi@ai.univ-paris8.fr
 * Date   : 02/10/2007
 */

#include "primitives.h"

/*
 * Couleur en cours
 */
static uint16 cur_color15 = 0x0;

/*
 * Modifier la couleur 15 bits en cours
 */
extern uint16 setColor15(uint16 c) {
  return cur_color15 = c;
}

/*
 * Renvoie la couleur 15 bits en cours
 */
extern uint16 getColor15(uint16 c) {
  return cur_color15;
}

/*
 * dessine un pixel avec cur_color dans VRAM_A
 */
extern void vramaPutPixel(int x, int y) {
  if(inScreen(x, y))
    VRAM_A[y * SCREEN_WIDTH + x] = cur_color15;
}

/*
 * drawLine dessine un segment (x0, y0) -> (x1, y1)
 * en utilisant l'algorithme de Bresenham'65.
 */
extern void drawLine(void (* ppfunc)(int x, int y), int x0, int y0, int x1, int y1) {
  int u = x1 - x0, v = y1 - y0;
  int pasX = (u < 0) ? -1 : 1, pasY = (v < 0) ? -1 : 1;
  int x, y, del, incH, incO;
  if(abs(u) < abs(v)) { /* deuxieme octant */
    del = (incH = ((pasX * u) << 1)) - pasY * v;
    incO = incH - ((pasY * v) << 1);
    for(y = y0, x = x0; y != y1; y += pasY) {
      ppfunc(x, y);
      if(del < 0) del += incH;
      else {
        del += incO;
        x += pasX;
      }
    }
  } else {  /* premier octant */
    del = (incH = ((pasY * v) << 1)) - pasX * u;
    incO = incH - ((pasX * u) << 1);
    for(x = x0, y = y0; x != x1; x += pasX) {
      ppfunc(x, y);
      if(del < 0) del += incH;
      else {
        del += incO;
        y += pasY;
      }
    }
  }
}
